

#include <stdio.h>

#include "defines.h"
#include "registers.h"
#include "tistdtypes.h"

#include <string.h>

#define ALLOC_EDMA_VARIABLES
#include "edma3.h"

Uint32          counterValStart,counterValStop,cpuCycles=0;

static void edmaWritePaRAM(UINT32 PaRAMSet, PaRAM * src, UINT32 nChannels);
static void edmaSetEvent(UINT32 channelNumber);


volatile PaRAM EDMA_DATA[EDMA_PaRAM_MAX];

enum RegID{
	OPT = 0,
	SRC,
	BA_CNT,
	DST,
	DS_BIDX,
	BCTRLD_LINK,
	DS_CIDX,
	CCNT
};


 UINT32 reg;

void EDMAInit(void)
{
    pEdmaECR   = (int *)(0x01c00000 + 0x1008);
    pEdmaESR   = (int *)(0x01c00000 + 0x1010);
    pEdmaSECR  = (int *)(0x01c00000 + 0x1040);
    pEdmaIER   = (int *)(0x01c00000 + 0x1050);
    pEdmaIECR  = (int *)(0x01c00000 + 0x1058);
    pEdmaIESR  = (int *)(0x01c00000 + 0x1060);
    pEdmaIPR   = (int *)(0x01c00000 + 0x1068);
    pEdmaICR   = (int *)(0x01c00000 + 0x1070);
    EDMA_RAM = (PaRAM *)EDMA_DATA;
    EDMA_ECR   = 0xffffffff; //  clear events  0 -> 31
    EDMA_SECR  = 0xffffffff; //  clear secondary events  0 -> 31
}

#define debugDetail
#undef debugDetail


Bool EdmaBlock1D(UINT32 EDMAChannelNo, UINT16* src, UINT16* dest, UINT32 acnt, UINT32 bcnt, UINT32 ndx)
{
   if (EDMAChannelNo >= EDMA_CHANNEL_MAX)
   {
       printf("Error in 'EDMABlock1D' - invalid DMA channel no., ch: %d, requested.\n", EDMAChannelNo); 
       return FALSE;
   }
   
   EDMA_RAM[EDMAChannelNo].Register[OPT] = 0;
   EDMA_RAM[EDMAChannelNo].Field.OPT_TCINTEN = 1;
   EDMA_RAM[EDMAChannelNo].Field.OPT_TCCHEN = 0;
   EDMA_RAM[EDMAChannelNo].Field.OPT_SYNCDIM=1;
   EDMA_RAM[EDMAChannelNo].Field.OPT_STATIC=1;
   EDMA_RAM[EDMAChannelNo].Field.OPT_TCC = EDMAChannelNo;
   EDMA_RAM[EDMAChannelNo].Register[SRC] = (UINT32)src;
   EDMA_RAM[EDMAChannelNo].Register[DST] = (UINT32)dest;
   EDMA_RAM[EDMAChannelNo].Field.ACNT = acnt;
   EDMA_RAM[EDMAChannelNo].Field.BCNT = bcnt;
   EDMA_RAM[EDMAChannelNo].Field.DSTBIDX = ndx;
   EDMA_RAM[EDMAChannelNo].Field.SRCBIDX = ndx;
   EDMA_RAM[EDMAChannelNo].Field.DSTCIDX = 0;
   EDMA_RAM[EDMAChannelNo].Field.SRCCIDX = 0;
   EDMA_RAM[EDMAChannelNo].Field.CCNT = 1;
   EDMA_RAM[EDMAChannelNo].Field.BCNTRLD = 0;
   EDMA_RAM[EDMAChannelNo].Field.LINK = 0xffff;
   
   edmaWritePaRAM(EDMAChannelNo, &EDMA_RAM[EDMAChannelNo], 1);  
       
   edmaSetEvent(EDMAChannelNo);  
                
   while( (reg = EDMA_IPR) == 0); //Poll for completion
        
   EdmaClearChannel(EDMAChannelNo); // Clear the IPR bit 
     
   return TRUE; 
} 




static void edmaCopyWords(UINT32 * dst, UINT32 * src, UINT32 count)
{
   while(count--)
   {
       *dst++ = *src++;
   }
}

static void edmaWritePaRAM(UINT32 PaRAMSet, PaRAM * src, UINT32 nSets)
{
   
   if (nSets > EDMA_CHANNEL_MAX )
   {
       return;
   }
   
   if (PaRAMSet > (EDMA_CHANNEL_MAX -1) )
   {
       return;
   }
   
   edmaCopyWords((UINT32 *)&((PaRAM *)EDMA_PARAM_BEGIN)[PaRAMSet], (UINT32 *)src, nSets * 8);   
}

   



static void edmaSetEvent(UINT32 channelNumber)
{
   int mask;
   
   if (channelNumber > (EDMA_CHANNEL_MAX -1) )
   {
       return;
   }

       mask = 1 << channelNumber;
       EDMA_ESR = mask;
}


void EdmaClearChannel(UINT32 channelNumber)
{
   int mask;

   if(channelNumber > (EDMA_CHANNEL_MAX -1) )
   {
       return;
   }
   
  
       mask = 1 << channelNumber;
       EDMA_ICR = mask;
    
}








void EdmaEnableChannel(UINT32 channelNumber, UINT32 QueueNumber)
{
   int mask;
   volatile int * regPtr;


   if(channelNumber > (EDMA_CHANNEL_MAX -1) )
   {
       return;
   }
   
       
       EDMA_IESR   =   1 << channelNumber;
       EDMA_DRAE0 |=   1 << channelNumber;  
       EDMA_DRAE1 &= ~(1 << channelNumber);
       EDMA_EMCR   =   1 << channelNumber;
       EDMA_SECR   =   1 << channelNumber;         
    
   QueueNumber &= 1; // only 0 or 1 are valid
   
   regPtr = (volatile int *)&EDMA_DMAQNUM0 + (channelNumber >> 3); // the offset is channel number divided by 8
   mask = 0x07 << (4 * (channelNumber & 0x07) );
   
   *regPtr &= ~mask;
   *regPtr |= QueueNumber << (4 * (channelNumber & 0x07) );        
   
}

void EdmaDisableChannel(UINT32 channelNumber)
{
   int mask;

   if(channelNumber > (EDMA_CHANNEL_MAX -1) )
   {
       return;
   }
   
       mask = 1 << channelNumber;
       EDMA_IECR |= mask;
       EDMA_DRAE0 &= ~mask;
   
}

