#include <stdio.h>
#include "edma3.h"
#include "registers.h"
#include "pattern.h"

#define ALLFs     0xFFFFFFFF
#define ALL0s     0x00000000
#define DDR_size 0x04000000  //64Mbyte
#define DDR_base 0xC0000000

#define TESTADDR1 0xC0004800
#define TESTADDR2 0xC001B400
#define TESTADDR3 0xC0009000
#define TESTADDR4 0xC0016C00

#define BUFFER1   0x11811000
#define BUFFER2   0x11812000
#define BUFFER3   0x11813000
#define BUFFER4   0x11814000

#define SYSCFG0_BASE_ADDR   0x01C14000
#define SYSCFG0_KICK0       *(unsigned int*)(SYSCFG0_BASE_ADDR + 0x38)
#define SYSCFG0_KICK1       *(unsigned int*)(SYSCFG0_BASE_ADDR + 0x3C)
#define SYSCFG0_CFGCHIP0    *(unsigned int*)(SYSCFG0_BASE_ADDR + 0x17C)

unsigned int M1failCount = 0, M3failCount = 0;
void pattern_testing(Uint16* pattern, Uint16 size, unsigned int count);
void verify_content(Uint8* ptr1, Uint8* ptr2, Uint16 size);

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  main( )                                                                 *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void main( void )
{
 unsigned int RoundCount =0;
  //Unlock KICK registers
  SYSCFG0_KICK0 = 0x83E70B13;
  SYSCFG0_KICK1 = 0x95A4F1E0;

  //Setup EDMA
  PSC0_lPSC_enable(0, 0); //Enable CC0
  PSC0_lPSC_enable(0, 1); //Enable TC0  
  EDMAInit();
  EdmaEnableChannel(31, 0);   
  SYSCFG0_CFGCHIP0 = (SYSCFG0_CFGCHIP0 & ~0x00000003) | 0x02; //64 byte burst size

  RoundCount = DDR_size/(sizeof(pattern_M1_M3));
  printf("\FIRST OF ALL, THIS IS FOR REFERENCE ONLY!!! 2016-Dec.-07.\n");
  printf("\DDR Stress Test with EDMA write/read DDR, compare result, Verified on L138 EVM Board.\n");
  printf("\For User Board, need to Change DDR_size defination According to your board to cover DDR address range.\n");
  printf("\The data patter target to increase DDR loading. If you want, can change data pattern.\n");
  printf("\If error found then your board's DDR is not stable, check DDR timing setting, and layout.\n");
  printf("\For product stability verification, it is better to run it under real working condition.\n");
  printf("\DDR blocks of pattern_M1_M3: %d\n", RoundCount);
  //  Run pattern
  pattern_testing((Uint16 *)pattern_M1_M3, sizeof(pattern_M1_M3), RoundCount);

  if(M1failCount + M3failCount == 0){
    printf("\nPASS. No errors found\n");
  } else {
    printf("\nFAIL. \nM1: %d errors \nM3: %d errors\n", M1failCount, M3failCount);
  }

  while(1); 
}


void pattern_testing(Uint16* pattern, Uint16 size, unsigned int RoundCount){

  volatile unsigned int *bufPtr1, *bufPtr2, *bufPtr3, *bufPtr4;
  volatile unsigned int *ddrPtr1, *ddrPtr2, *ddrPtr3, *ddrPtr4;
  unsigned int j, n;


  for(n= 0; n<RoundCount; n++){
	  EdmaBlock1D(31, (Uint16 *)pattern, (Uint16 *)(DDR_base + n * size), size, 1, 1);
	  EdmaBlock1D(31, (Uint16 *)(DDR_base + n * size), (Uint16 *)BUFFER1, size, 1, 1);
	  verify_content ((Uint8 *)pattern, (Uint8 *) BUFFER1, size);
  }

#if 0
  ddrPtr1 = (volatile unsigned int *) TESTADDR1;
  ddrPtr2 = (volatile unsigned int *) TESTADDR2;
  ddrPtr3 = (volatile unsigned int *) TESTADDR3;
  ddrPtr4 = (volatile unsigned int *) TESTADDR4;

  bufPtr1 = (volatile unsigned int *) BUFFER1;
  bufPtr2 = (volatile unsigned int *) BUFFER2;
  bufPtr3 = (volatile unsigned int *) BUFFER3;
  bufPtr4 = (volatile unsigned int *) BUFFER4;

  for(j=0; j<size*2; j++) {
    *ddrPtr1++ = ALLFs;
    *ddrPtr2++ = ALLFs;
    *ddrPtr3++ = ALLFs;
    *ddrPtr4++ = ALLFs;

    *bufPtr1++ = ALL0s;
    *bufPtr2++ = ALL0s;
    *bufPtr3++ = ALL0s;  
    *bufPtr4++ = ALL0s;
  }

  //Write to DDR
  EdmaBlock1D(31, (Uint16 *)pattern, (Uint16 *)TESTADDR1, size, 1, 1);
  EdmaBlock1D(31, (Uint16 *)pattern, (Uint16 *)TESTADDR2, size, 1, 1);
  EdmaBlock1D(31, (Uint16 *)pattern, (Uint16 *)TESTADDR3, size, 1, 1);
  EdmaBlock1D(31, (Uint16 *)pattern, (Uint16 *)TESTADDR4, size, 1, 1);

  //Read from DDR
  EdmaBlock1D(31, (Uint16 *)TESTADDR1, (Uint16 *)BUFFER1, size, 1, 1);
  EdmaBlock1D(31, (Uint16 *)TESTADDR2, (Uint16 *)BUFFER2, size, 1, 1);
  EdmaBlock1D(31, (Uint16 *)TESTADDR3, (Uint16 *)BUFFER3, size, 1, 1);
  EdmaBlock1D(31, (Uint16 *)TESTADDR4, (Uint16 *)BUFFER4, size, 1, 1);

  //Verify contents
  verify_content((Uint8 *)pattern, (Uint8 *) BUFFER1, size);
  verify_content((Uint8 *)pattern, (Uint8 *) BUFFER2, size);
  verify_content((Uint8 *)pattern, (Uint8 *) BUFFER3, size);
  verify_content((Uint8 *)pattern, (Uint8 *) BUFFER4, size);
#endif
}

void verify_content(Uint8* ptr1, Uint8* ptr2, Uint16 size){

  unsigned int j,k;
#if 0
  for(k=0; k<5; k++){
    for(j=0; j<64; j++) {
      if(*ptr1 != *ptr2){		
        M1failCount++;
      }
      ptr1++;
      ptr2++;
    }
    for(j=0; j<128; j++){
      if(*ptr1 != *ptr2){
        M3failCount++;
      }
  	  ptr1++;
	  ptr2++;
    }
  }
#endif
  for(j=0; j<size; j++){
	  if(*ptr1 != *ptr2){
	         M1failCount++;
	  }
  }

}
