
#ifndef _REGISTERS_H_
#define _REGISTERS_H_

#include "defines.h"

#define RegisterSetBits(R, n) \
{\
	volatile int temp = *(volatile int *)(R); \
	temp |= (n); \
	*(volatile int *)(R) = (temp);\
}

#define RegisterClearBits(R, n) \
{\
	volatile int temp = *(volatile int *)(R); \
	temp &= ~(n); \
	*(volatile int *)(R) = (temp);\
}

#define RegisterSetValue(R, n) \
{\
	*(volatile int *)(R) = (n);\
}


#define RegisterMerge(R, n, mask) \
{\
	volatile int temp = *(volatile int *)(R); \
	temp &= ~(mask); \
	temp |= (n & mask); \
	*(volatile int *)(R) = (temp);\
}



/*
	volatile int temp = *(volatile int *)(R); \
	temp &= ~(mask); \
	temp |= (n & mask);\
	*(volatile int *)(R) = (temp);\	*/




// interrupt control registers
#define INTC_BASE	(0x01c48000)


#define  EDMACC_BASE		(0x01c00000)
#define  EDMA_PARAM_BEGIN 	(EDMACC_BASE + 0x4000)
#define  QDMA_CCCFG			(EDMACC_BASE + 0x0004)

#define  QDMA_CHANNELMAP0	(EDMACC_BASE + 0x0200)

#define  EDMA_DMAQNUM0			( *(volatile int *)(EDMACC_BASE + 0x0240))  	// DMA Queue Number Reg 0
#define  EDMA_DMAQNUM1			( *(volatile int *)(EDMACC_BASE + 0x0244))  	// DMA Queue Number Reg 1
#define  EDMA_DMAQNUM2			( *(volatile int *)(EDMACC_BASE + 0x0248))  	// DMA Queue Number Reg 2
#define  EDMA_DMAQNUM3			( *(volatile int *)(EDMACC_BASE + 0x024c))  	// DMA Queue Number Reg 3
#define  EDMA_DMAQNUM4			( *(volatile int *)(EDMACC_BASE + 0x0250))  	// DMA Queue Number Reg 4
#define  EDMA_DMAQNUM5			( *(volatile int *)(EDMACC_BASE + 0x0254))  	// DMA Queue Number Reg 5
#define  EDMA_DMAQNUM6			( *(volatile int *)(EDMACC_BASE + 0x0258))  	// DMA Queue Number Reg 6
#define  EDMA_DMAQNUM7			( *(volatile int *)(EDMACC_BASE + 0x025c))  	// DMA Queue Number Reg 7
#define  EDMA_QDMAQNUM			( *(volatile int *)(EDMACC_BASE + 0x0260))  	// QDMA Queue Number Reg


#define  EDMA_EMR			( *(volatile int *)(EDMACC_BASE + 0x0300))  	// event missed Reg
#define  EDMA_EMCR			( *(volatile int *)(EDMACC_BASE + 0x0308))  	// event missed Clear Reg
#define  EDMA_QEMR			( *(volatile int *)(EDMACC_BASE + 0x0310)) 	 	// QDMA event missed Reg
#define  EDMA_QEMCR			( *(volatile int *)(EDMACC_BASE + 0x0314))  	// QDMA event missed Clear Reg 
#define  EDMA_CCERR			( *(volatile int *)(EDMACC_BASE + 0x0318))  	// EDMA3CC Error Reg
#define  EDMA_CCERRCLR		( *(volatile int *)(EDMACC_BASE + 0x031C))  	// EDMA3CC Error Clear Reg


#define  EDMA_DRAE0			( *(volatile int *)(EDMACC_BASE + 0x340))  // DMA access region0
#define  EDMA_DRAE1			( *(volatile int *)(EDMACC_BASE + 0x348))  // DMA access region1



#define  EDMA_ECR			( *(volatile int *)(EDMACC_BASE + 0x1008))  // event clear Reg
#define  EDMA_ESR			( *(volatile int *)(EDMACC_BASE + 0x1010))  // event set Reg
#define  EDMA_SECR			( *(volatile int *)(EDMACC_BASE + 0x1040))  // secondary event clear Reg
#define  EDMA_IER			( *(volatile int *)(EDMACC_BASE + 0x1050))  // interrupt enable Reg
#define  EDMA_IECR			( *(volatile int *)(EDMACC_BASE + 0x1058))  // interrupt disable Reg
#define  EDMA_IESR			( *(volatile int *)(EDMACC_BASE + 0x1060))  // interrupt enable Reg
#define  EDMA_ICR			( *(volatile int *)(EDMACC_BASE + 0x1070))  // interrupt clear Reg
#define  EDMA_IPR			( *(volatile int *)(EDMACC_BASE + 0x1068))  // interrupt pending Reg

#endif //#ifndef _REGISTERS_H_

